@@ -35,6 +35,7 @@ class MedianFilter:
                - "nearest": aaa|abcd|ddd
                - "wrap": bcd|abcd|abc
                - "constant": 000|abcd|000
+
             Default is "reflect".
         threshold: float, optional
             Threshold for the "thresholded median filter".
@@ -124,9 +125,7 @@ class MedianFilter:
         self.cuda_kernel_2d = self._module.get_function("medfilt2d")
         # Blocks, grid
         self._block_size = {2: (32, 32, 1), 3: (16, 8, 8)}[self.data_ndim]  # TODO tune
-        self._n_blocks = tuple(
-            [updiv(a, b) for a, b in zip(self.shape[::-1], self._block_size)]
-        )
+        self._n_blocks = tuple([updiv(a, b) for a, b in zip(self.shape[::-1], self._block_size)])
 
     def medfilt2(self, image, output=None):
         """
