@@ -4,8 +4,10 @@ from .utils import get_cuda_context, __has_pycuda__
 if __has_pycuda__:
     import pycuda.driver as cuda
     import pycuda.gpuarray as garray
+
     dev_attrs = cuda.device_attribute
 
+
 # NB: we must detach from a context before creating another context
 class CudaProcessing:
     def __init__(self, device_id=None, ctx=None, stream=None, cleanup_at_exit=True):
@@ -30,10 +32,7 @@ class CudaProcessing:
             Ignored if ctx is not None.
         """
         if ctx is None:
-            self.ctx = get_cuda_context(
-                device_id=device_id,
-                cleanup_at_exit=cleanup_at_exit
-            )
+            self.ctx = get_cuda_context(device_id=device_id, cleanup_at_exit=cleanup_at_exit)
         else:
             self.ctx = ctx
         self.stream = stream
@@ -42,16 +41,13 @@ class CudaProcessing:
         self.device_id = self.device.get_attribute(dev_attrs.MULTI_GPU_BOARD_GROUP_ID)
         self._allocated = {}
 
-
     def push_context(self):
         self.ctx.push()
         return self.ctx
 
-
     def pop_context(self):
         self.ctx.pop()
 
-
     def init_arrays_to_none(self, arrays_names):
         """
         Initialize arrays to None. After calling this method, the current instance will
@@ -67,7 +63,6 @@ class CudaProcessing:
             setattr(self, "_old_" + array_name, None)
             self._allocated[array_name] = False
 
-
     def recover_arrays_references(self, arrays_names):
         """
         Performs self._array_name = self._old_array_name,
@@ -79,11 +74,10 @@ class CudaProcessing:
             List of array names
         """
         for array_name in arrays_names:
-            old_arr = getattr(self, "_old_" + array_name)
+            old_arr = getattr(self, "_old_" + array_name, None)
             if old_arr is not None:
                 setattr(self, array_name, old_arr)
 
-
     def allocate_array(self, array_name, shape, dtype=np.float32):
         """
         Allocate a GPU array on the current context/stream/device,
@@ -104,7 +98,6 @@ class CudaProcessing:
             self._allocated[array_name] = True
         return getattr(self, array_name)
 
-
     def set_array(self, array_name, array_ref, shape, dtype=np.float32):
         """
         Set the content of a device array.
@@ -121,17 +114,35 @@ class CudaProcessing:
             Data type. Default is float32.
         """
         if isinstance(array_ref, garray.GPUArray):
-            current_arr = getattr(self, array_name)
+            current_arr = getattr(self, array_name, None)
             setattr(self, "_old_" + array_name, current_arr)
             setattr(self, array_name, array_ref)
         elif isinstance(array_ref, np.ndarray):
-            self._allocate_array(array_name, shape, dtype=dtype)
+            self.allocate_array(array_name, shape, dtype=dtype)
             getattr(self, array_name).set(array_ref)
         else:
             raise ValueError("Expected numpy array or pycuda array")
 
+    def get_array(self, array_name):
+        return getattr(self, array_name, None)
+
     # COMPAT.
     _init_arrays_to_none = init_arrays_to_none
     _recover_arrays_references = recover_arrays_references
     _allocate_array = allocate_array
     _set_array = set_array
+
+    @staticmethod
+    def check_array(arr, expected_shape, expected_dtype="f", check_contiguous=True):
+        """
+        Check whether a given array is suitable for being processed (shape, dtype, contiguous)
+        """
+        if arr.shape != expected_shape:
+            raise ValueError("Expected shape %s but got %s" % (str(expected_shape), str(arr.shape)))
+        if arr.dtype != np.dtype(expected_dtype):
+            raise ValueError("Expected data type %s but got %s" % (str(expected_dtype), str(arr.dtype)))
+        if check_contiguous:
+            if isinstance(arr, np.ndarray) and not (arr.flags["C_CONTIGUOUS"]):
+                raise ValueError("Expected C-contiguous array")
+            if isinstance(arr, garray.GPUArray) and not arr.flags.c_contiguous:
+                raise ValueError("Expected C-contiguous array")
