@@ -2,12 +2,14 @@ import atexit
 from math import ceil
 import numpy as np
 from ..resources.gpu import GPUDescription
+
 try:
     import pycuda
     import pycuda.driver as cuda
     from pycuda import gpuarray as garray
     from pycuda.tools import make_default_context
     from pycuda.tools import clear_context_caches
+
     __has_pycuda__ = True
     __pycuda_error_msg__ = None
     if pycuda.VERSION[0] < 2020:
@@ -18,6 +20,7 @@ except ImportError as err:
     __pycuda_error_msg__ = str(err)
 try:
     import skcuda
+
     __has_cufft__ = True
 except ImportError:
     __has_cufft__ = False
@@ -42,12 +45,14 @@ def get_cuda_context(device_id=None, cleanup_at_exit=True):
     # Unlike Context.make_context(), the newly-created context is not made current.
     context = cuda.Device(device_id).retain_primary_context()
     context.push()
+
     # Register a clean-up function at exit
     def _finish_up(context):
         if context is not None:
             context.pop()
             context = None
         clear_context_caches()
+
     if cleanup_at_exit:
         atexit.register(_finish_up, context)
     return context
@@ -64,7 +69,7 @@ def get_gpu_memory(device_id):
     Return the total memory (in GigaBytes) of a device.
     """
     cuda.init()
-    return cuda.Device(device_id).total_memory()/1e9
+    return cuda.Device(device_id).total_memory() / 1e9
 
 
 def is_gpu_usable():
@@ -78,7 +83,7 @@ def is_gpu_usable():
         if exc.__str__() != "cuInit failed: no CUDA-capable device is detected":
             raise
         n_gpus = 0
-    res = (n_gpus > 0)
+    res = n_gpus > 0
     return res
 
 
@@ -96,7 +101,7 @@ def detect_cuda_gpus():
     """
     gpus = {}
     error_msg = None
-    if not(__has_pycuda__):
+    if not (__has_pycuda__):
         return {}, __pycuda_error_msg__
     try:
         cuda.init()
@@ -141,6 +146,8 @@ double precision: pycuda-helpers.hpp:
   typedef uint2 fp_tex_cfloat;  // --> complex64
   typedef int4 fp_tex_cdouble;  // --> complex128
 """
+
+
 def cuarray_format_to_dtype(cuarr_fmt):
     # reverse of cuda.dtype_to_array_format
     fmt = cuda.array_format
@@ -162,7 +169,7 @@ def cuarray_shape_dtype(cuarray):
     desc = cuarray.get_descriptor_3d()
     shape = (desc.height, desc.width)
     if desc.depth > 0:
-        shape = (desc.depth, ) + shape
+        shape = (desc.depth,) + shape
     dtype = cuarray_format_to_dtype(desc.format)
     return shape, dtype
 
@@ -214,14 +221,14 @@ def copy_array(dst, src, check=False, src_dtype=None):
         copy.set_src_array(src)
     elif isinstance(src, garray.GPUArray):
         copy.set_src_device(src.gpudata)
-    else: # numpy
+    else:  # numpy
         copy.set_src_host(src)
 
     if isinstance(dst, cuda.Array):
         copy.set_dst_array(dst)
     elif isinstance(dst, garray.GPUArray):
         copy.set_dst_device(dst.gpudata)
-    else: # numpy
+    else:  # numpy
         copy.set_dst_host(dst)
 
     copy.width_in_bytes = copy.dst_pitch = w * np.dtype(dtype_src).itemsize
@@ -234,7 +241,6 @@ def copy_array(dst, src, check=False, src_dtype=None):
     ###
 
 
-
 def copy_big_gpuarray(dst, src, itemsize=4, checks=False):
     """
     Copy a big `pycuda.gpuarray.GPUArray` into another.
@@ -252,21 +258,23 @@ def copy_big_gpuarray(dst, src, itemsize=4, checks=False):
         else:
             dst[:] = src[:]
         return
+
     def get_shape2(shape):
         shape2 = list(shape)
-        while np.prod(shape2)*4 > limit:
+        while np.prod(shape2) * 4 > limit:
             shape2[0] //= 2
         return tuple(shape2)
+
     shape2 = get_shape2(dst.shape)
     nz0 = dst.shape[0]
     nz = shape2[0]
     n_transfers = ceil(nz0 / nz)
     for i in range(n_transfers):
-        zmax = min((i+1)*nz, nz0)
+        zmax = min((i + 1) * nz, nz0)
         if d2h:
-            src[i*nz:zmax].get(ary=dst[i*nz:zmax])
+            src[i * nz : zmax].get(ary=dst[i * nz : zmax])
         else:
-            dst[i*nz:zmax] = src[i*nz:zmax]
+            dst[i * nz : zmax] = src[i * nz : zmax]
 
 
 def replace_array_memory(arr, new_shape):
@@ -280,4 +288,3 @@ def replace_array_memory(arr, new_shape):
     arr.shape = new_shape
     # TODO re-compute strides
     return arr
-
