@@ -37,6 +37,8 @@ local_ifftn = scipy.fft.irfftn
 
 
 class AlignmentBase:
+    default_extra_options = {"blocking_plots": False}
+
     def __init__(
         self,
         vert_fft_width=False,
@@ -44,6 +46,7 @@ class AlignmentBase:
         verbose=False,
         logger=None,
         data_type=np.float32,
+        extra_options=None,
     ):
         """
         Alignment basic functions.
@@ -66,10 +69,10 @@ class AlignmentBase:
             Computation data type.
         """
 
-        self._init_parameters(vert_fft_width, horz_fft_width, verbose, logger, data_type)
+        self._init_parameters(vert_fft_width, horz_fft_width, verbose, logger, data_type, extra_options=extra_options)
         self._plot_windows = {}
 
-    def _init_parameters(self, vert_fft_width, horz_fft_width, verbose, logger, data_type):
+    def _init_parameters(self, vert_fft_width, horz_fft_width, verbose, logger, data_type, extra_options=None):
         self.logger = LoggerOrPrint(logger)
         self.truncate_vert_pow2 = vert_fft_width
         self.truncate_horz_pow2 = horz_fft_width
@@ -79,6 +82,8 @@ class AlignmentBase:
             verbose = False
         self.verbose = verbose
         self.data_type = data_type
+        self.extra_options = self.default_extra_options.copy()
+        self.extra_options.update(extra_options or {})
 
     @staticmethod
     def _check_img_stack_size(img_stack: np.ndarray, img_pos: np.ndarray):
@@ -202,7 +207,7 @@ class AlignmentBase:
         return vertex_yx
 
     @staticmethod
-    def refine_max_position_1d(f_vals, fx=None, return_vertex_val=False):
+    def refine_max_position_1d(f_vals, fx=None, return_vertex_val=False, return_all_coeffs=False):
         """Computes the sub-pixel max position of the given function sampling.
 
         Parameters
@@ -254,7 +259,9 @@ class AlignmentBase:
 
         # For a 1D parabola `f(x) = c + bx + ax^2`, the vertex position is:
         # x_v = -b / 2a.
-        vertex_x = (-coeffs[1, :] / (2 * coeffs[2, :]))[0]
+        vertex_x = -coeffs[1, :] / (2 * coeffs[2, :])
+        if not return_all_coeffs:
+            vertex_x = vertex_x[0]
 
         vertex_min_x = np.min(fx)
         vertex_max_x = np.max(fx)
