@@ -4,7 +4,7 @@ from ..misc import fourier_filters
 from .alignment import AlignmentBase, plt, progress_bar, local_fftn, local_ifftn
 
 # three possible  values for the validity check, which can optionally be returned by the find_shifts methods
-cor_result_validity =  {
+cor_result_validity = {
     "unknown": "unknown",
     "sound": "sound",
     "correct": "sound",
@@ -34,10 +34,10 @@ class CenterOfRotation(AlignmentBase):
         The output of this function, allows to compute motor movements for
         aligning the sample rotation axis. Given the following values:
 
-        - L1: distance from source to motor
-        - L2: distance from source to detector
-        - ps: physical pixel size
-        - v: output of this function
+           - L1: distance from source to motor
+           - L2: distance from source to detector
+           - ps: physical pixel size
+           - v: output of this function
 
         displacement of motor = (L1 / L2 * ps) * v
 
@@ -236,9 +236,7 @@ class CenterOfRotationSlidingWindow(CenterOfRotation):
         self._check_img_pair_sizes(img_1, img_2)
 
         if peak_fit_radius < 1:
-            self.logger.warning(
-                "Parameter peak_fit_radius should be at least 1, given: %d instead." % peak_fit_radius
-            )
+            self.logger.warning("Parameter peak_fit_radius should be at least 1, given: %d instead." % peak_fit_radius)
             peak_fit_radius = 1
 
         img_shape = img_2.shape
@@ -267,7 +265,9 @@ class CenterOfRotationSlidingWindow(CenterOfRotation):
         elif side.lower() == "center":
             win_2_start = img_shape[-1] // 2 - window_shift
         else:
-            raise ValueError("Side should be one of 'left', 'right', or 'center'. '%s' was given instead" % side.lower())
+            raise ValueError(
+                "Side should be one of 'left', 'right', or 'center'. '%s' was given instead" % side.lower()
+            )
 
         win_2_end = win_2_start + window_width
 
@@ -473,7 +473,9 @@ class CenterOfRotationGrowingWindow(CenterOfRotation):
             win_2_mid_start = -img_upper_half_size + min_window_width
             win_2_mid_end = np.ceil(img_shape[-1] * 3 / 2).astype(np.intp) - min_window_width
         else:
-            raise ValueError("Side should be one of 'left', 'right', or 'center'. '%s' was given instead" % side.lower())
+            raise ValueError(
+                "Side should be one of 'left', 'right', or 'center'. '%s' was given instead" % side.lower()
+            )
 
         n1 = win_1_mid_end - win_1_mid_start
         n2 = win_2_mid_end - win_2_mid_start
@@ -701,8 +703,13 @@ class CenterOfRotationAdaptiveSearch(CenterOfRotation):
         found_centers = []
         x_cor = lim_1
         while x_cor < lim_2:
-
-            tmp_sigma = min((img_1.shape[1] - x_cor), (x_cor),) * self.sigma_fraction
+            tmp_sigma = (
+                min(
+                    (img_1.shape[1] - x_cor),
+                    (x_cor),
+                )
+                * self.sigma_fraction
+            )
 
             tmp_x = (np.arange(img_1.shape[1]) - x_cor) / tmp_sigma
             apodis = np.exp(-tmp_x * tmp_x / 2.0)
@@ -721,12 +728,9 @@ class CenterOfRotationAdaptiveSearch(CenterOfRotation):
                     roi_yxhw=roi_yxhw,
                 )
             except ValueError as err:
-                message = "ValueError from base class CenterOfRotation.find_shift in CenterOfRotationAdaptiveSearch.find_shift  : {err}".format(
-                    err=err
-                )
-                self.logger.warning(message)
-                x_cor = min(x_cor + x_cor * self.step_fraction, x_cor + (dim_radio - x_cor) * self.step_fraction)
-                continue
+                if "positions are outside the input margins" in str(err):
+                    x_cor = min(x_cor + x_cor * self.step_fraction, x_cor + (dim_radio - x_cor) * self.step_fraction)
+                    continue
             except:
                 message = "Unexpected error from base class CenterOfRotation.find_shift in CenterOfRotationAdaptiveSearch.find_shift  : {err}".format(
                     err=err
@@ -746,7 +750,13 @@ class CenterOfRotationAdaptiveSearch(CenterOfRotation):
                 cor_position = p_2 / 2
 
             cor_in_img = img_1.shape[1] // 2 + cor_position
-            tmp_sigma = min((img_1.shape[1] - cor_in_img), (cor_in_img),) * self.sigma_fraction
+            tmp_sigma = (
+                min(
+                    (img_1.shape[1] - cor_in_img),
+                    (cor_in_img),
+                )
+                * self.sigma_fraction
+            )
 
             M1 = int(round(cor_position + img_1.shape[1] // 2)) - int(round(tmp_sigma))
             M2 = int(round(cor_position + img_1.shape[1] // 2)) + int(round(tmp_sigma))
