@@ -1,16 +1,19 @@
 import os
-from typing import Union
+from math import ceil
+from multiprocessing.pool import ThreadPool
 import numpy as np
 from silx.io.dictdump import h5todict
 from tomoscan.io import HDF5File
 from .utils import get_compacted_dataslices, convert_dict_values
-from ..misc.binning import get_binning_function
-from ..utils import subsample_dict, get_3D_subregion
+from ..misc.binning import binning as image_binning
+from ..utils import subsample_dict, get_3D_subregion, get_num_threads
+
 try:
     from silx.third_party.EdfFile import EdfFile
 except ImportError:
     EdfFile = None
 
+
 class Reader:
     """
     Abstract class for various file readers.
@@ -44,14 +47,12 @@ class Reader:
             self.start_y = 0
             self.end_y = None
 
-
     def get_data(self, data_url):
         """
         Get data from a silx.io.url.DataUrl
         """
         raise ValueError("Base class")
 
-
     def release(self):
         """
         Release the file if needed.
@@ -60,7 +61,6 @@ class Reader:
 
 
 class NPReader(Reader):
-
     multi_load = True
 
     def __init__(self, sub_region=None, mmap=True):
@@ -99,9 +99,9 @@ class NPReader(Reader):
         data_ref = self._open(data_url)
         data_slice = data_url.data_slice()
         if data_slice is None:
-            res = data_ref[self.start_y:self.end_y, self.start_x:self.end_x]
+            res = data_ref[self.start_y : self.end_y, self.start_x : self.end_x]
         else:
-            res = data_ref[data_slice, self.start_y:self.end_y, self.start_x:self.end_x]
+            res = data_ref[data_slice, self.start_y : self.end_y, self.start_x : self.end_x]
         return res
 
     def release(self):
@@ -115,8 +115,7 @@ class NPReader(Reader):
 
 
 class EDFReader(Reader):
-
-    multi_load = False # not implemented
+    multi_load = False  # not implemented
 
     def __init__(self, sub_region=None):
         """
@@ -145,7 +144,6 @@ class EDFReader(Reader):
 
 
 class HDF5Reader(Reader):
-
     multi_load = True
 
     def __init__(self, sub_region=None):
@@ -155,21 +153,20 @@ class HDF5Reader(Reader):
         super().__init__(sub_region=sub_region)
         self._file_desc = {}
 
-
     def _open(self, file_path):
         if file_path not in self._file_desc:
             self._file_desc[file_path] = HDF5File(file_path, "r", swmr=True)
 
-
     def get_data(self, data_url):
         file_path = data_url.file_path()
         self._open(file_path)
         h5dataset = self._file_desc[file_path][data_url.data_path()]
         data_slice = data_url.data_slice()
         if data_slice is None:
-            res = h5dataset[self.start_y:self.end_y, self.start_x:self.end_x]
+            res = h5dataset[self.start_y : self.end_y, self.start_x : self.end_x]
         else:
-            res = h5dataset[data_slice, self.start_y:self.end_y, self.start_x:self.end_x]
+            res = h5dataset[data_slice, self.start_y : self.end_y, self.start_x : self.end_x]
+
         return res
 
     def release(self):
@@ -189,11 +186,14 @@ class HDF5Loader:
     """
     An alternative class to HDF5Reader where information is first passed at class instantiation
     """
-    def __init__(self, fname, data_path, sub_region=None, pre_allocate=True, dtype="f"):
+
+    def __init__(self, fname, data_path, sub_region=None, data_buffer=None, pre_allocate=True, dtype="f"):
         self.fname = fname
         self.data_path = data_path
         self._set_subregion(sub_region)
-        self.data = None
+        if not ((data_buffer is not None) ^ (pre_allocate is True)):
+            raise ValueError("Please provide either 'data_buffer' or 'pre_allocate'")
+        self.data = data_buffer
         self._loaded = False
         if pre_allocate:
             expected_shape = get_hdf5_dataset_shape(fname, data_path, sub_region=sub_region)
@@ -211,15 +211,18 @@ class HDF5Loader:
             self.start_y, self.end_y = None, None
             self.start_z, self.end_z = None, None
 
-
     def load_data(self, force_load=False):
         if self._loaded and not force_load:
             return self.data
         with HDF5File(self.fname, "r") as fdesc:
             if self.data is None:
-                self.data = fdesc[self.data_path][self.start_z:self.end_z, self.start_y:self.end_y, self.start_x:self.end_x]
+                self.data = fdesc[self.data_path][
+                    self.start_z : self.end_z, self.start_y : self.end_y, self.start_x : self.end_x
+                ]
             else:
-                self.data[:] = fdesc[self.data_path][self.start_z:self.end_z, self.start_y:self.end_y, self.start_x:self.end_x]
+                self.data[:] = fdesc[self.data_path][
+                    self.start_z : self.end_z, self.start_y : self.end_y, self.start_x : self.end_x
+                ]
         self._loaded = True
         return self.data
 
@@ -232,14 +235,16 @@ class ChunkReader:
     def __init__(
         self,
         files,
-        sub_region: Union[tuple, None] = None,
+        sub_region=None,
+        detector_corrector=None,
         pre_allocate=True,
         data_buffer=None,
         convert_float=False,
         shape=None,
         dtype=None,
         binning=None,
-        dataset_subsampling=None
+        dataset_subsampling=None,
+        num_threads=None,
     ):
         """
         Initialize a "ChunkReader". A chunk is a stack of images.
@@ -277,6 +282,10 @@ class ChunkReader:
         dataset_subsampling: int, optional
             Whether to subsample the dataset. If an integer `n` is provided,
             then one image out of `n` will be read.
+        num_threads: int, optional
+            Number of threads to use for binning the data.
+            Default is to use all available threads.
+            This parameter has no effect when binning is disabled.
 
         Notes
         ------
@@ -287,8 +296,10 @@ class ChunkReader:
         the data retrieval (quite the opposite), since the whole (subregion of) data
         is read and then binning is performed.
         """
+        self.detector_corrector = detector_corrector
         self._get_reader_class(files)
         self.dataset_subsampling = dataset_subsampling
+        self.num_threads = get_num_threads(num_threads)
         self._set_files(files)
         self._get_shape_and_dtype(shape, dtype, binning)
         self._set_subregion(sub_region)
@@ -299,19 +310,20 @@ class ChunkReader:
             self.out_dtype = np.float32
         else:
             self.out_dtype = self.dtype
-        if not((data_buffer is not None) ^ (pre_allocate is True)):
+        if not ((data_buffer is not None) ^ (pre_allocate is True)):
             raise ValueError("Please provide either 'data_buffer' or 'pre_allocate'")
         self.files_data = data_buffer
         if data_buffer is not None:
             # overwrite out_dtype
             self.out_dtype = data_buffer.dtype
-            if data_buffer.shape != self.shape:
+            if data_buffer.shape != self.chunk_shape:
                 raise ValueError("Expected shape %s but got %s" % (self.shape, data_buffer.shape))
         if pre_allocate:
             self.files_data = np.zeros(self.chunk_shape, dtype=self.out_dtype)
         if (self.binning is not None) and (np.dtype(self.out_dtype).kind in ["u", "i"]):
-            raise ValueError("Output datatype cannot be integer when using binning. Please set the 'convert_float' parameter to True or specify a 'data_buffer'.")
-
+            raise ValueError(
+                "Output datatype cannot be integer when using binning. Please set the 'convert_float' parameter to True or specify a 'data_buffer'."
+            )
 
     def _set_files(self, files):
         if len(files) == 0:
@@ -322,15 +334,11 @@ class ChunkReader:
         self._fileindex_to_idx = dict.fromkeys(self._sorted_files_indices)
         self._configure_subsampling()
 
-
     def _infer_file_type(self, files):
         fname = files[sorted(files.keys())[0]].file_path()
         ext = os.path.splitext(fname)[-1].replace(".", "")
         if ext not in Readers:
-            raise ValueError(
-                "Unknown file format %s. Supported formats are: %s"
-                % (ext, str(Readers.keys()))
-            )
+            raise ValueError("Unknown file format %s. Supported formats are: %s" % (ext, str(Readers.keys())))
         return ext
 
     def _get_reader_class(self, files):
@@ -338,7 +346,6 @@ class ChunkReader:
         reader_class = Readers[ext]
         self._reader_class = reader_class
 
-
     def _get_shape_and_dtype(self, shape, dtype, binning):
         if shape is None or dtype is None:
             shape, dtype = self._infer_shape_and_dtype()
@@ -347,21 +354,19 @@ class ChunkReader:
         self.dtype = dtype
         self._set_binning(binning)
 
-
     def _configure_subsampling(self):
         dataset_subsampling = self.dataset_subsampling
         self.files_subsampled = self.files
         if dataset_subsampling is not None and dataset_subsampling > 1:
-            self.n_files //= dataset_subsampling
             self.files_subsampled = subsample_dict(self.files, dataset_subsampling)
-            if not(self._reader_class.multi_load):
+            self.n_files = len(self.files_subsampled)
+            if not (self._reader_class.multi_load):
                 # 3D loading not supported for this reader.
                 # Data is loaded frames by frame, so subsample directly self.files
                 self.files = self.files_subsampled
                 self._sorted_files_indices = sorted(self.files.keys())
                 self._fileindex_to_idx = dict.fromkeys(self._sorted_files_indices)
 
-
     def _infer_shape_and_dtype(self):
         self._reader_entire_image = self._reader_class()
         first_file_dataurl = self.files[self._sorted_files_indices[0]]
@@ -384,13 +389,17 @@ class ChunkReader:
         if self.binning is not None:
             self.shape = (self.shape[0] // self.binning[1], self.shape[1] // self.binning[0])
         self.chunk_shape = (self.n_files,) + self.shape
+        if self.detector_corrector is not None:
+            self.detector_corrector.set_sub_region_transformation(target_sub_region=self.sub_region)
 
     def _init_reader(self):
         # instantiate reader with user params
-        self.file_reader = self._reader_class(
-            sub_region=self.sub_region
-        )
+        if self.detector_corrector is not None:
+            adapted_subregion = self.detector_corrector.get_adapted_subregion(self.sub_region)
+        else:
+            adapted_subregion = self.sub_region
 
+        self.file_reader = self._reader_class(sub_region=adapted_subregion)
 
     def _set_binning(self, binning):
         if binning is None:
@@ -406,33 +415,52 @@ class ChunkReader:
         for b in binning:
             if int(b) != b:
                 raise ValueError("Expected an integer number for binning values, but got %s" % binning)
-            # Current limitation
-            if b not in [1, 2, 3]:
-                raise NotImplementedError("Currently, binning can only be 2 or 3 in each dimension")
-            #
         self.binning = binning
-        binning_function = get_binning_function(binning[::-1]) # user provides (binning_x, binning_y)
-        if binning_function is None:
-            raise NotImplementedError("Binning factor %s is not implemented yet" % str(binning))
-        self.binning_function = binning_function
-
 
     def get_data(self, file_url):
         """
         Get the data associated to a file url.
         """
         arr = self.file_reader.get_data(file_url)
-        if self.binning is not None:
-            if arr.ndim == 2:
-                arr = self.binning_function(arr)
-            else:
-                nz = arr.shape[0]
-                res = np.zeros((nz, ) + self.binning_function(arr[0]).shape, dtype="f")
-                for i in range(nz):
-                    res[i] = self.binning_function(arr[i])
-                arr = res
-        return arr
 
+        if arr.ndim == 2:
+            if self.detector_corrector is not None:
+                arr = self.detector_corrector.transform(arr)
+            if self.binning is not None:
+                arr = image_binning(arr, self.binning[::-1])
+
+        else:
+            if self.detector_corrector is not None:
+                if self.detector_corrector is not None:
+                    _, (
+                        src_x_start,
+                        src_x_end,
+                        src_z_start,
+                        src_z_end,
+                    ) = self.detector_corrector.get_actual_shapes_source_target()
+
+                    arr_target = np.empty([len(arr), src_z_end - src_z_start, src_x_end - src_x_start], "f")
+
+                def apply_corrector(i_img_tuple):
+                    i, img = i_img_tuple
+                    arr_target[i] = self.detector_corrector.transform(img)
+
+                with ThreadPool(self.num_threads) as tp:
+                    tp.map(apply_corrector, enumerate(arr))
+                arr = arr_target
+            if self.binning is not None:
+                nz = arr.shape[0]
+                res = np.zeros((nz,) + image_binning(arr[0], self.binning[::-1]).shape, dtype="f")
+
+                def apply_binning(img_res_tuple):
+                    img, res = img_res_tuple
+                    res[:] = image_binning(img, self.binning[::-1])
+
+                with ThreadPool(self.num_threads) as tp:
+                    tp.map(apply_binning, zip(arr, res))
+                arr = res
+
+        return arr
 
     def _load_single(self):
         for i, fileidx in enumerate(self._sorted_files_indices):
@@ -440,7 +468,6 @@ class ChunkReader:
             self.files_data[i] = self.get_data(file_url)
             self._fileindex_to_idx[fileidx] = i
 
-
     def _load_multi(self):
         urls_compacted = get_compacted_dataslices(self.files, subsampling=self.dataset_subsampling)
         loaded = {}
@@ -454,14 +481,13 @@ class ChunkReader:
             ds = url.data_slice()
             delta_z = ds.stop - ds.start
             if ds.step is not None and ds.step > 1:
-                delta_z //= ds.step
+                delta_z = ceil(delta_z / ds.step)
             end_idx = start_idx + delta_z
             self.files_data[start_idx:end_idx] = self.get_data(url)
             start_idx += delta_z
             loaded[url_str] = True
 
-
-    def load_files(self, overwrite: bool=False):
+    def load_files(self, overwrite: bool = False):
         """
         Load the files whose links was provided at class instantiation.
 
@@ -470,7 +496,7 @@ class ChunkReader:
         overwrite: bool, optional
             Whether to force reloading the files if already loaded.
         """
-        if self._loaded and not(overwrite):
+        if self._loaded and not (overwrite):
             raise ValueError("Radios were already loaded. Call load_files(overwrite=True) to force reloading")
         if self.file_reader.multi_load:
             self._load_multi()
@@ -553,7 +579,7 @@ def get_hdf5_dataset_shape(fname, h5_data_path, sub_region=None):
     # perhaps there is more elegant
     res_shape = []
     for n, bounds in zip([n_z, n_y, n_x], ((zmin, zmax), (ymin, ymax), (xmin, xmax))):
-        res_shape.append(np.arange(n)[bounds[0]:bounds[1]].size)
+        res_shape.append(np.arange(n)[bounds[0] : bounds[1]].size)
     return tuple(res_shape)
 
 
@@ -590,9 +616,5 @@ def import_h5_to_dict(h5file, h5path, asarray=False):
         Whether to convert each numeric value to an 0D array. Default is False.
     """
     dic = h5todict(h5file, path=h5path, asarray=asarray)
-    modified_dic = convert_dict_values(
-        dic,
-        {"None": None},
-        bytes_tostring=True
-    )
+    modified_dic = convert_dict_values(dic, {"None": None}, bytes_tostring=True)
     return modified_dic
