@@ -20,15 +20,21 @@ nabu_config = {
             "validator": float_validator,
             "type": "advanced",
         },
+        "darks_flats_dir": {
+            "default": "",
+            "help": "Path to a directory where XXX_flats.h5 and XXX_darks.h5 are to be found, where 'XXX' denotes the dataset basename. If these files are found, then reduced flats/darks will be loaded from them. Otherwise, reduced flats/darks will be saved to there once computed, either in the .nx directory, or in the output directory. Mind that the HDF5 entry corresponds to the one of the dataset.",
+            "validator": optional_directory_location_validator,
+            "type": "optional",
+        },
         "binning": {
             "default": "1",
-            "help": "Binning factor in the horizontal dimension when reading the data.\nThe final slices dimensions will be divided by 'slices_binning'",
+            "help": "Binning factor in the horizontal dimension when reading the data.\nThe final slices dimensions will be divided by this factor.",
             "validator": binning_validator,
             "type": "advanced",
         },
         "binning_z": {
             "default": "1",
-            "help": "Binning factor in the vertical dimension when reading the data.\nThis retsults in a lesser number of reconstructed slices.",
+            "help": "Binning factor in the vertical dimension when reading the data.\nThis results in a lesser number of reconstructed slices.",
             "validator": binning_validator,
             "type": "advanced",
         },
@@ -44,11 +50,17 @@ nabu_config = {
             "validator": optional_file_location_validator,
             "type": "advanced",
         },
+        "overwrite_metadata": {
+            "default": "",
+            "help": "Which metadata to overwrite, separated by a semicolon, and with units. Example: 'energy = 19 kev; pixel_size = 1.6 um'",
+            "validator": no_validator,
+            "type": "advanced",
+        },
     },
     "preproc": {
         "flatfield": {
             "default": "1",
-            "help": "How to perform flat-field normalization. The parameter value can be:\n - 1 or True: enabled.\n - 0 or False: disabled\n - forced or force-load: perform flatfield regardless of the dataset by attempting to load darks/flats from 'xxx_tomwer_processes.h5' file or 'xxx_nabu_processes.h5' or the file provided in parameter 'processes_file'\n - force-compute: perform flatfield, ignore all .h5 files containing already computed darks/flats.",
+            "help": "How to perform flat-field normalization. The parameter value can be:\n - 1 or True: enabled.\n - 0 or False: disabled\n - forced or force-load: perform flatfield regardless of the dataset by attempting to load darks/flats\n - force-compute: perform flatfield, ignore all .h5 files containing already computed darks/flats.",
             "validator": flatfield_enabled_validator,
             "type": "required",
         },
@@ -64,6 +76,12 @@ nabu_config = {
             "validator": optional_string_validator,
             "type": "advanced",
         },
+        "normalize_srcurrent": {
+            "default": "0",
+            "help": "Whether to normalize frames with Synchrotron Current. This can correct the effect of a beam refill not taken into account by flats.",
+            "validator": boolean_validator,
+            "type": "advanced",
+        },
         "ccd_filter_enabled": {
             "default": "0",
             "help": "Whether to enable the CCD hotspots correction.",
@@ -76,6 +94,18 @@ nabu_config = {
             "validator": float_validator,
             "type": "optional",
         },
+        "detector_distortion_correction": {
+            "default": "",
+            "help": "Apply coordinate transformation on the raw data, at the reading stage. Default (empty) is None. Available are: None, identity(for testing the pipeline), map_xz. This latter method requires two URLs being passet by detector_distortion_correction_options: map_x and map_z pointing to two 2D arrays containing the position where each pixel can be interpolated at in the raw data",
+            "validator": detector_distortion_correction_validator,
+            "type": "advanced",
+        },
+        "detector_distortion_correction_options": {
+            "default": "",
+            "help": "Options for sinogram rings correction methods. The parameters are separated by commas and passed as 'name=value', for example: center_xz=(1000,100); angle_deg=5. Mind the semicolon separator (;).",
+            "validator": generic_options_validator,
+            "type": "advanced",
+        },
         "double_flatfield_enabled": {
             "default": "0",
             "help": "Whether to enable the 'double flat-field' filetering for correcting rings artefacts.",
@@ -108,10 +138,16 @@ nabu_config = {
         },
         "sino_normalization": {
             "default": "",
-            "help": "Sinogram normalization method. Available methods are: chebyshev, none. Default is none (no normalization)",
+            "help": "Sinogram normalization method. Available methods are: chebyshev, subtraction, division, none. Default is none (no normalization)",
             "validator": sino_normalization_validator,
             "type": "advanced",
         },
+        "sino_normalization_file": {
+            "default": "",
+            "help": "Path to the file when sino_normalization is either 'subtraction' or 'division'. To specify the path within a HDF5 file, the syntax is /path/to/file?path=/entry/data",
+            "validator": no_validator,
+            "type": "advanced",
+        },
         "processes_file": {
             "default": "",
             "help": "Path to the file where some operations should be stored for later use. By default it is 'xxx_nabu_processes.h5'",
@@ -127,7 +163,7 @@ nabu_config = {
         "sino_rings_options": {
             "default": "sigma=1.0 ; levels=10",
             "help": "Options for sinogram rings correction methods. The parameters are separated by commas and passed as 'name=value', for example: sigma=1.0;levels=10. Mind the semicolon separator (;).",
-            "validator": cor_options_validator,
+            "validator": generic_options_validator,
             "type": "advanced",
         },
         "rotate_projections": {
@@ -154,7 +190,6 @@ nabu_config = {
             "validator": generic_options_validator,
             "type": "advanced",
         },
-
     },
     "phase": {
         "method": {
@@ -169,12 +204,6 @@ nabu_config = {
             "validator": float_validator,
             "type": "required",
         },
-        "margin": {
-            "default": "50",
-            "help": "Margin (in pixels) in the Paganin filtering.",
-            "validator": integer_validator,
-            "type": "advanced",
-        },
         "unsharp_coeff": {
             "default": "0",
             "help": "Unsharp mask strength. The unsharped image is equal to\n  UnsharpedImage =  (1 + coeff)*originalPaganinImage - coeff * ConvolvedImage. Setting this coefficient to zero means that no unsharp mask will be applied.",
@@ -189,7 +218,7 @@ nabu_config = {
         },
         "unsharp_method": {
             "default": "gaussian",
-            "help": "Which type of unsharp mask filter to use. Available values are gaussian (UnsharpedImage =  (1 + coeff)*originalPaganinImage - coeff * ConvolvedImage) and laplacian (UnsharpedImage = originalPaganinImage + coeff * ConvolvedImage). Default is gaussian.",
+            "help": "Which type of unsharp mask filter to use. Available values are gaussian, laplacian and imagej. Default is gaussian.",
             "validator": unsharp_method_validator,
             "type": "optional",
         },
@@ -205,12 +234,6 @@ nabu_config = {
             "validator": optional_string_validator,
             "type": "optional",
         },
-        "ctf_translations_file": {
-            "default": "",
-            "help": "A file where each line has two values: horizontal and vertical displacements. There should be as many values as there are projection images.",
-            "validator": optional_file_location_validator,
-            "type": "advanced",
-        },
         "ctf_advanced_params": {
             "default": "length_scale=1e-5; lim1=1e-5; lim2=0.2; normalize_by_mean=True",
             "help": "Advanced parameters for CTF phase retrieval.",
@@ -221,7 +244,7 @@ nabu_config = {
     "reconstruction": {
         "method": {
             "default": "FBP",
-            "help": "Reconstruction method. Possible values: FBP, none. If value is 'none', no reconstruction will be done.",
+            "help": "Reconstruction method. Possible values: FBP, cone, none. If value is 'none', no reconstruction will be done.",
             "validator": reconstruction_method_validator,
             "type": "required",
         },
@@ -232,15 +255,15 @@ nabu_config = {
             "type": "optional",
         },
         "rotation_axis_position": {
-            "default": "",
-            "help": "Rotation axis position. Default (empty) is the middle of the detector width.\nAdditonally, the following methods are available to find automaticall the Center of Rotation (CoR):\n - centered : a fast and simple auto-CoR method. It only works when the CoR is not far from the middle of the detector. It does not work for half-tomography.\n - global : a slow but robust auto-CoR.\n - sliding-window : semi-automatically find the CoR with a sliding window. You have to specify on which side the CoR is (left, center, right). Please see the 'cor_options' parameter.\n - growing-window : automatically find the CoR with a sliding-and-growing window. You can tune the option with the parameter 'cor_options'.\n - sino-coarse-to-fine: Estimate CoR from sinogram. Only works for 360 degrees scans.\n - composite-coarse-to-fine: Estimate CoR from composite multi-angle images. Only works for 360 degrees scans.",
+            "default": "sliding-window",
+            "help": "Rotation axis position. It can be a number or the name of an estimation method (empty value means the middle of the detector).\nThe following methods are available to find automatically the Center of Rotation (CoR):\n - centered : a fast and simple auto-CoR method. It only works when the CoR is not far from the middle of the detector. It does not work for half-tomography.\n - global : a slow but robust auto-CoR.\n - sliding-window : semi-automatically find the CoR with a sliding window. You have to specify on which side the CoR is (left, center, right). Please see the 'cor_options' parameter.\n - growing-window : automatically find the CoR with a sliding-and-growing window. You can tune the option with the parameter 'cor_options'.\n - sino-coarse-to-fine: Estimate CoR from sinogram. Only works for 360 degrees scans.\n - composite-coarse-to-fine: Estimate CoR from composite multi-angle images. Only works for 360 degrees scans.",
             "validator": cor_validator,
             "type": "required",
         },
         "cor_options": {
             "default": "",
             "help": "Options for methods finding automatically the rotation axis position. The parameters are separated by commas and passed as 'name=value', for example: low_pass=1; high_pass=20. Mind the semicolon separator (;).",
-            "validator": cor_options_validator,
+            "validator": generic_options_validator,
             "type": "advanced",
         },
         "cor_slice": {
@@ -257,7 +280,7 @@ nabu_config = {
         },
         "translation_movements_file": {
             "default": "",
-            "help": "A file where each line describes the translation in X and Z of the sample (or detector).",
+            "help": "A file where each line describes the horizontal and vertical translations of the sample (or detector). The order is 'horizontal, vertical'.",
             "validator": optional_values_file_validator,
             "type": "advanced",
         },
@@ -269,15 +292,33 @@ nabu_config = {
         },
         "fbp_filter_type": {
             "default": "ramlak",
-            "help": "Filter type for FBP method. Available are: ramlak, none",
+            "help": "Filter type for FBP method. Available are: none, ramlak, shepp-logan, cosine, hamming, hann, tukey, lanczos, hilbert",
             "validator": fbp_filter_name_validator,
             "type": "advanced",
         },
+        "fbp_filter_cutoff": {
+            "default": "1.",
+            "help": "Cut-off frequency for Fourier filter used in FBP, in normalized units. Default is the Nyquist frequency 1.0",
+            "validator": float_validator,
+            "type": "advanced",
+        },
+        "source_sample_dist": {
+            "default": "",
+            "help": "In cone-beam geometry, distance (in meters) between the X-ray source and the center of the sample. Default is infinity.",
+            "validator": optional_float_validator,
+            "type": "advanced",
+        },
+        "sample_detector_dist": {
+            "default": "",
+            "help": "In cone-beam geometry, distance (in meters) between the center of the sample and the detector. Default is read from the input dataset.",
+            "validator": optional_float_validator,
+            "type": "advanced",
+        },
         "padding_type": {
-            "default": "zeros",
+            "default": "edges",
             "help": "Padding type for FBP. Available are: zeros, edges",
             "validator": padding_mode_validator,
-            "type": "optional", # put "advanced" with default value "edges" ?
+            "type": "optional",  # put "advanced" with default value "edges" ?
         },
         "enable_halftomo": {
             "default": "auto",
@@ -299,7 +340,7 @@ nabu_config = {
         },
         "start_x": {
             "default": "0",
-            "help": "\nParameters for sub-volume reconstruction. Indices start at 0 !\n----------------------------------------------------------------\n(x, y) are the dimension of a slice, and (z) is the 'vertical' axis\nBy default, all the volume is reconstructed slice by slice, along the axis 'z'.",
+            "help": "\nParameters for sub-volume reconstruction. Indices start at 0, and upper bounds are INCLUDED!\n----------------------------------------------------------------\n(x, y) are the dimension of a slice, and (z) is the 'vertical' axis\nBy default, all the volume is reconstructed slice by slice, along the axis 'z'.",
             "validator": nonnegative_integer_validator,
             "type": "optional",
         },
@@ -379,12 +420,12 @@ nabu_config = {
         },
         "file_format": {
             "default": "hdf5",
-            "help": "Output file format. Available are: hdf5, tiff, jp2, edf",
+            "help": "Output file format. Available are: hdf5, tiff, jp2, edf, vol",
             "validator": output_file_format_validator,
             "type": "optional",
         },
         "overwrite_results": {
-            "default": "0",
+            "default": "1",
             "help": "What to do in the case where the output file exists.\nBy default, the output data is never overwritten and the process is interrupted if the file already exists.\nSet this option to 1 if you want to overwrite the output files.",
             "validator": boolean_validator,
             "type": "required",
@@ -499,7 +540,7 @@ nabu_config = {
         },
     },
     # This section will be removed in the future (for now it is deprecated)
-    "about": {}
+    "about": {},
 }
 
 renamed_keys = {
@@ -543,5 +584,4 @@ renamed_keys = {
         "since": "2021.2.0",
         "message": "Option 'flatfield_enabled' has been renamed 'flatfield' in [preproc]",
     },
-
 }
