@@ -1,26 +1,11 @@
 import numpy as np
-from ..utils import check_supported, deprecated_class
-from scipy.ndimage.filters import median_filter
-
-# COMPAT.
-from .flatfield import FlatField as Flatfield_, FlatFieldArrays as FlatFieldArrays_, FlatFieldDataUrls as FlatFieldDataUrls_
-FlatField = deprecated_class(
-    "preproc.ccd.FlatField was moved to preproc.flatfield.FlatField", do_print=True
-)(Flatfield_)
-FlatFieldArrays = deprecated_class(
-    "preproc.ccd.FlatFieldArrays was moved to preproc.flatfield.FlatFieldArrays", do_print=True
-)(FlatFieldArrays_)
-FlatFieldDataUrls = deprecated_class(
-    "preproc.ccd.FlatFieldDataUrls was moved to preproc.flatfield.FlatFieldDataUrls", do_print=True
-)(FlatFieldDataUrls_)
-#
+from ..utils import check_supported
+from scipy.ndimage import median_filter
 
 
 class CCDFilter:
     """
-    A base class for processings applied on radios.
-    It is designed to handle chunks of radios (see the documentation for more
-    information on the radios chunk concept).
+    Filtering applied on radios.
     """
 
     _supported_ccd_corrections = ["median_clip"]
@@ -69,7 +54,6 @@ class CCDFilter:
         self.abs_diff = abs_diff
         self.preserve_borders = preserve_borders
 
-
     def _set_radios_shape(self, radios_shape):
         if len(radios_shape) == 2:
             self.radios_shape = (1,) + radios_shape
@@ -141,10 +125,6 @@ class CCDFilter:
         return output
 
 
-# Compat.
-CCDCorrection = deprecated_class("CCDCorrection is replaced with CCDFilter", do_print=True)(CCDFilter)
-
-
 class Log:
     """
     Helper class to take -log(radios)
