@@ -1,7 +1,6 @@
-import numpy as np
 from .double_flatfield import DoubleFlatField
 from ..utils import check_shape
-from ..cuda.utils import get_cuda_context, __has_pycuda__
+from ..cuda.utils import __has_pycuda__
 from ..cuda.processing import CudaProcessing
 from ..misc.unsharp_cuda import CudaUnsharpMask
 
@@ -9,13 +8,14 @@ if __has_pycuda__:
     import pycuda.gpuarray as garray
     import pycuda.cumath as cumath
 
-class CudaDoubleFlatField(DoubleFlatField):
 
+class CudaDoubleFlatField(DoubleFlatField):
     def __init__(
         self,
         shape,
         result_url=None,
         sub_region=None,
+        detector_corrector=None,
         input_is_mlog=True,
         output_is_mlog=False,
         average_is_on_log=False,
@@ -31,23 +31,19 @@ class CudaDoubleFlatField(DoubleFlatField):
             shape,
             result_url=result_url,
             sub_region=sub_region,
+            detector_corrector=detector_corrector,
             input_is_mlog=input_is_mlog,
             output_is_mlog=output_is_mlog,
             average_is_on_log=average_is_on_log,
             sigma_filter=sigma_filter,
-            filter_mode=filter_mode
+            filter_mode=filter_mode,
         )
         self._init_gaussian_filter()
 
-
     def _init_gaussian_filter(self):
         if self.sigma_filter is None:
             return
-        self._unsharp_mask = CudaUnsharpMask(
-            self.shape, self.sigma_filter, -1.,
-            mode=self.filter_mode, method="log"
-        )
-
+        self._unsharp_mask = CudaUnsharpMask(self.shape, self.sigma_filter, -1.0, mode=self.filter_mode, method="log")
 
     @staticmethod
     def _proc_copy(x, o):
@@ -64,14 +60,13 @@ class CudaDoubleFlatField(DoubleFlatField):
     @staticmethod
     def _proc_mlog(x, o, min_clip=None):
         if min_clip is not None:
-            garray.maximum(x, min_clip, output=o)
+            garray.maximum(x, min_clip, out=o)
             cumath.log(o, out=o)
         else:
             cumath.log(x, out=o)
         o *= -1
         return o
 
-
     def _init_processing(self, input_is_mlog, output_is_mlog, average_is_on_log, sigma_filter, filter_mode):
         self.input_is_mlog = input_is_mlog
         self.output_is_mlog = output_is_mlog
@@ -105,7 +100,6 @@ class CudaDoubleFlatField(DoubleFlatField):
         self.proc = proc
         self.postproc = postproc
 
-
     def compute_double_flatfield(self, radios, recompute=False):
         """
         Read the radios and generate the "double flat field" by averaging
@@ -118,9 +112,9 @@ class CudaDoubleFlatField(DoubleFlatField):
         recompute: bool, optional
             Whether to recompute the double flatfield if already computed.
         """
-        if not(isinstance(radios, garray.GPUArray)):
+        if not (isinstance(radios, garray.GPUArray)):
             raise ValueError("Expected pycuda.gpuarray.GPUArray for radios")
-        if self._computed:
+        if self._computed and not (recompute):
             return self.doubleflatfield
         acc = garray.zeros(radios[0].shape, "f")
         tmpdat = garray.zeros(radios[0].shape, "f")
@@ -139,20 +133,14 @@ class CudaDoubleFlatField(DoubleFlatField):
         # Handle small values to avoid issues when dividing
         # self.doubleflatfield[np.abs(self.doubleflatfield) < self._small] = 1.
         cumath.fabs(self.doubleflatfield, out=acc)
-        acc -= self._small # acc = abs(doubleflatfield) - _small
-        garray.if_positive(
-            acc,
-            self.doubleflatfield,
-            garray.zeros_like(acc) + self._small,
-            out=self.doubleflatfield
-        )
+        acc -= self._small  # acc = abs(doubleflatfield) - _small
+        garray.if_positive(acc, self.doubleflatfield, garray.zeros_like(acc) + self._small, out=self.doubleflatfield)
 
         if self.writer is not None:
             self.writer.write(self.doubleflatfield.get(), "double_flatfield")
         self._computed = True
         return self.doubleflatfield
 
-
     def get_double_flatfield(self, radios=None, compute=False):
         """
         Get the double flat field or a subregion of it.
@@ -167,9 +155,7 @@ class CudaDoubleFlatField(DoubleFlatField):
         """
         if self.reader is None:
             if radios is None:
-                raise ValueError(
-                    "result_url was not provided. Please provide 'radios' to this function"
-                )
+                raise ValueError("result_url was not provided. Please provide 'radios' to this function")
             return self.compute_double_flatfield(radios)
 
         if radios is not None and compute:
@@ -180,7 +166,6 @@ class CudaDoubleFlatField(DoubleFlatField):
             self._computed = True
         return res
 
-
     def apply_double_flatfield(self, radios):
         """
         Apply the "double flatfield" filter on a chunk of radios.
