@@ -3,10 +3,12 @@ import numpy as np
 from scipy.ndimage import shift as ndshift
 from nabu.preproc.shift import VerticalShift
 from nabu.cuda.utils import __has_pycuda__, get_cuda_context
+
 if __has_pycuda__:
     from nabu.preproc.shift_cuda import CudaVerticalShift, garray
 
-@pytest.fixture(scope='class')
+
+@pytest.fixture(scope="class")
 def bootstrap(request):
     cls = request.cls
     data = np.zeros([13, 11], "f")
@@ -23,9 +25,7 @@ def bootstrap(request):
         padded_radio = np.concatenate(
             [cls.radios[iradio], np.zeros([1, 11], "f")], axis=0
         )  # needs padding because ndshifs does not work as expected
-        shifted_padded_radio = ndshift(
-            padded_radio, [-my_shift, 0], mode="constant", cval=0.0, order=1
-        ).astype("f")
+        shifted_padded_radio = ndshift(padded_radio, [-my_shift, 0], mode="constant", cval=0.0, order=1).astype("f")
         shifted_radio = shifted_padded_radio[:-1]
         golden.append(shifted_radio)
     cls.golden = np.array(golden)
@@ -33,7 +33,8 @@ def bootstrap(request):
     if __has_pycuda__:
         cls.ctx = get_cuda_context()
 
-@pytest.mark.usefixtures('bootstrap')
+
+@pytest.mark.usefixtures("bootstrap")
 class TestVerticalShift:
     def test_vshift(self):
         radios = self.radios.copy()
@@ -44,12 +45,10 @@ class TestVerticalShift:
         Shifter.apply_vertical_shifts(radios, self.indexes, output=new_radios)
         assert abs(new_radios - self.golden).max() < self.tol
 
-
         Shifter.apply_vertical_shifts(radios, self.indexes)
         assert abs(radios - self.golden).max() < self.tol
 
-
-    @pytest.mark.skipif(not(__has_pycuda__), reason="Need cuda/pycuda for this test")
+    @pytest.mark.skipif(not (__has_pycuda__), reason="Need cuda/pycuda for this test")
     def test_cuda_vshift(self):
         d_radios = garray.to_gpu(self.radios)
         d_out = garray.zeros_like(d_radios)
